<?php

namespace Uncanny_Automator_Pro;

/**
 * Class BDB_SETUSERPROFILEDATA
 *
 * @package Uncanny_Automator_Pro
 */
class BDB_SETUSERPROFILEDATA {

	/**
	 * Integration code
	 *
	 * @var string
	 */
	public static $integration = 'BDB';

	private $action_code;
	private $action_meta;

	/**
	 * SetAutomatorTriggers constructor.
	 */
	public function __construct() {
		$this->action_code = 'BDBSETUSERPROFILEDATA';
		$this->action_meta = 'BDBPROFILE';

		$this->define_action();
	}

	/**
	 * Define and register the action by pushing it into the Automator object.
	 */
	public function define_action() {

		$action = array(
			'author'             => Automator()->get_author_name(),
			'support_link'       => Automator()->get_author_support_link( $this->action_code, 'integration/buddyboss/' ),
			'is_pro'             => true,
			'integration'        => self::$integration,
			'code'               => $this->action_code,
			/* translators: Action - BuddyBoss */
			'sentence'           => sprintf( __( "Set the user's {{Xprofile data:%1\$s}}", 'uncanny-automator-pro' ), $this->action_meta ),
			/* translators: Action - BuddyBoss */
			'select_option_name' => __( "Set the user's {{Xprofile data}}", 'uncanny-automator-pro' ),
			'priority'           => 10,
			'accepted_args'      => 1,
			'execution_function' => array( $this, 'update_user_profile_data' ),
			'options_callback'   => array( $this, 'load_options' ),
		);

		Automator()->register->action( $action );
	}

	/**
	 * @return array[]
	 */
	public function load_options() {
		return Automator()->utilities->keep_order_of_options(
			array(
				'options_group' => array(
					$this->action_meta => array(
						array(
							'input_type'        => 'repeater',
							'option_code'       => 'BDBPROFILEDATA',
							'label'             => '',
							'required'          => true,
							'fields'            => array(
								Automator()->helpers->recipe->buddyboss->options->pro->list_all_profile_fields( esc_attr__( 'Field', 'uncanny-automator-pro' ), 'BDBUSERFIELD', array( 'is_repeater' => true ) ),
								array(
									'input_type'      => 'text',
									'option_code'     => 'VALUE',
									'label'           => esc_attr__( 'Value', 'uncanny-automator-pro' ),
									'supports_tokens' => true,
									'required'        => false,
								),
							),

							/* translators: Non-personal infinitive verb */
							'add_row_button'    => esc_attr__( 'Add pair', 'uncanny-automator-pro' ),
							/* translators: Non-personal infinitive verb */
							'remove_row_button' => esc_attr__( 'Remove pair', 'uncanny-automator-pro' ),
						),
					),
				),
			)
		);
	}

	/**
	 * Update user profile type
	 *
	 * @param string $user_id
	 * @param array $action_data
	 * @param string $recipe_id
	 *
	 * @since 1.1
	 * @return void
	 *
	 */
	public function update_user_profile_data( $user_id, $action_data, $recipe_id, $args ) {

		$user_fields_data = $action_data['meta']['BDBPROFILEDATA'];

		// Adding other users
		if ( ! empty( $user_fields_data ) ) {
			$user_selectors = json_decode( $user_fields_data, true );
			if ( ! empty( $user_selectors ) ) {
				foreach ( $user_selectors as $user_selector ) {
					$field_id = $user_selector['BDBUSERFIELD'];
					if ( ! empty( $user_selector['VALUE'] ) ) {
						$value = Automator()->parse->text( $user_selector['VALUE'], $recipe_id, $user_id, $args );
						if ( function_exists( 'xprofile_set_field_data' ) ) {
							xprofile_set_field_data( $field_id, $user_id, $value );
						}
					}
				}
			}
		}
		Automator()->complete_action( $user_id, $action_data, $recipe_id );
	}

}
