<?php
namespace Uncanny_Automator_Pro;

/**
 * Class ET_ANON_TOKENS
 *
 * @package Uncanny_Automator_Pro
 */
class ET_ANON_TOKENS {

	public function __construct() {

		add_filter(
			'automator_maybe_trigger_ec_attendeeregistered_tokens',
			array(
				$this,
				'et_anonattendee_possible_tokens',
			),
			20,
			2
		);

		add_filter( 'automator_maybe_parse_token', array( $this, 'et_anonattendee_tokens' ), 20, 6 );

	}

	public function et_anonattendee_possible_tokens( $tokens = array(), $args = array() ) {

		if ( ! automator_pro_do_identify_tokens() ) {
			return $tokens;
		}

		$trigger_integration = $args['integration'];
		$trigger_meta        = $args['meta'];

		$fields = array(
			array(
				'tokenId'         => 'holder_name',
				'tokenName'       => __( 'Attendee name', 'uncanny-automator-pro' ),
				'tokenType'       => 'text',
				'tokenIdentifier' => $trigger_meta,
			),
			array(
				'tokenId'         => 'holder_email',
				'tokenName'       => __( 'Attendee email', 'uncanny-automator-pro' ),
				'tokenType'       => 'text',
				'tokenIdentifier' => $trigger_meta,
			),
		);

		$tokens = array_merge( $tokens, $fields );

		return $tokens;
	}

	public function et_anonattendee_tokens( $value, $pieces, $recipe_id, $trigger_data, $user_id, $replace_args ) {

		if ( $pieces ) {

			if ( in_array( 'ATTENDEEREGISTERED', $pieces, true ) ) {

				$to_replace = $pieces[2];
				$event_id   = Automator()->db->token->get( 'event_id', $replace_args );
				$event      = get_post( $event_id );

				if ( ! $event instanceof \WP_Post ) {
					return $value;
				}

				switch ( $to_replace ) {
					case 'ANONATTENDEE':
						$value = esc_html( $event->post_title );
						break;
					case 'ANONATTENDEE_URL':
						$value = get_permalink( $event->ID );
						break;
					case 'ANONATTENDEE_THUMB_ID':
						$value = get_post_thumbnail_id( $event->ID );
						break;
					case 'ANONATTENDEE_THUMB_URL':
						$value = get_the_post_thumbnail_url( $event->ID );
						break;
					case 'holder_name':
						$value = Automator()->db->token->get( 'holder_name', $replace_args );
						$value = maybe_unserialize( $value );
						break;
					case 'holder_email':
						$value = Automator()->db->token->get( 'holder_email', $replace_args );
						$value = maybe_unserialize( $value );
						break;
					default:
						$value = $event->ID;
						break;
				}
			}
		}

		return $value;
	}

}
