<?php

namespace Uncanny_Automator_Pro;

use Uncanny_Automator\Recipe\Actions;

/**
 * Class LD_EMAILGROUPLEADERS
 *
 * @package Uncanny_Automator_Pro
 */
class LD_EMAILGROUPLEADERS {

	use Actions;

	/**
	 * Reset key holder
	 *
	 * @var null
	 */
	private $key;

	/**
	 * Set up Automator action constructor.
	 */
	public function __construct() {

		$this->key_generated = false;

		$this->key = null;

		$this->setup_action();
	}

	/**
	 * Setting up the action
	 *
	 * @return void
	 */
	protected function setup_action() {

		$this->set_integration( 'LD' );

		$this->set_action_code( 'LD_EMAILGROUPLEADERS' );

		$this->set_action_meta( 'LD_EMAILGROUPLEADERS_META' );

		$this->set_requires_user( true );

		$this->set_is_pro( true );

		$this->set_sentence(
			sprintf(
			/* translators: Action - WordPress */
				esc_attr__( 'Send an {{email:%1$s}} to Group Leaders of {{a group:%2$s}}', 'uncanny-automator-pro' ),
				$this->get_action_meta(),
				$this->get_action_meta() . '_GROUP_ID'
			)
		);

		/* translators: Action - WordPress */
		$this->set_readable_sentence( esc_attr__( 'Send an {{email}} to Group Leaders of {{a group}}', 'uncanny-automator' ) );

		$this->set_options_callback( array( $this, 'load_options' ) );

		$this->register_action();

	}

	/**
	 * @return array
	 */
	public function load_options() {

		$options_group = array(
			$this->get_action_meta() => array(
				// Email From Field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code' => 'EMAILFROM',
						/* translators: Email field */
						'label'       => esc_attr__( 'From', 'uncanny-automator' ),
						'input_type'  => 'email',
						'default'     => '{{admin_email}}',
					)
				),

				// Email From Field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code' => 'EMAILFROMNAME',
						/* translators: Email field */
						'label'       => esc_attr__( 'From name', 'uncanny-automator' ),
						'input_type'  => 'text',
						'default'     => '{{site_name}}',
					)
				),

				// Email CC field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code' => 'EMAILCC',
						/* translators: Email field */
						'label'       => esc_attr__( 'CC', 'uncanny-automator-pro' ),
						'input_type'  => 'email',
						'required'    => false,
					)
				),

				// Email BCC field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code' => 'EMAILBCC',
						/* translators: Email field */
						'label'       => esc_attr__( 'BCC', 'uncanny-automator-pro' ),
						'input_type'  => 'email',
						'required'    => false,
					)
				),

				// Email Subject field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code' => 'EMAILSUBJECT',
						/* translators: Email field */
						'label'       => esc_attr__( 'Subject', 'uncanny-automator-pro' ),
						'required'    => true,
					)
				),

				// Email Content Field.
				Automator()->helpers->recipe->field->text(
					array(
						'option_code'               => 'EMAILBODY',
						/* translators: Email field */
						'label'                     => esc_attr__( 'Body', 'uncanny-automator-pro' ),
						'input_type'                => 'textarea',
						'supports_fullpage_editing' => true,
					)
				),

			),
		);

		return Automator()->utilities->keep_order_of_options(
			array(
				'options'       => array(
					array(
						'option_code'              => $this->get_action_meta() . '_GROUP_ID',
						/* translators: Email field */
						'label'                    => esc_attr__( 'Group', 'uncanny-automator-pro' ),
						'input_type'               => 'select',
						'supports_tokens'          => true,
						'supports_custom_value'    => true,
						'custom_value_description' => esc_attr__( 'Group ID', 'uncanny-automator-pro' ),
						'required'                 => true,
						'options'                  => $this->get_learndash_groups(),
					),
				),
				'options_group' => $options_group,
			)
		);
	}

	/**
	 * Handling action function
	 *
	 * @param $user_id
	 * @param $action_data
	 * @param $recipe_id
	 * @param $args
	 * @param $parsed
	 *
	 * @return void.
	 */
	protected function process_action( $user_id, $action_data, $recipe_id, $args, $parsed ) {

		$body_text = isset( $parsed['EMAILBODY'] ) ? $parsed['EMAILBODY'] : '';

		$group_id = isset( $parsed[ $this->get_action_meta() . '_GROUP_ID' ] ) ? absint( $parsed[ $this->get_action_meta() . '_GROUP_ID' ] ) : 0;

		if ( false !== strpos( $body_text, '{{reset_pass_link}}' ) ) {

			$reset_pass = ! is_null( $this->key ) ? $this->key : Automator()->parse->generate_reset_token( $user_id );

			$body = str_replace( '{{reset_pass_link}}', $reset_pass, $body_text );

		} else {

			$body = $body_text;

		}

		$group_admin_emails = $this->get_group_admin_emails( $group_id );
		if ( empty( $group_admin_emails ) ) {
			$error_message = __( 'The selected group does not have a Group Leader.', 'uncanny-automator-pro' );

			$action_data['complete_with_errors'] = true;

			Automator()->complete->action( $user_id, $action_data, $recipe_id, $error_message );

			return;
		}

		$data = array(
			'to'        => $group_admin_emails,
			'from'      => isset( $parsed['EMAILFROM'] ) ? $parsed['EMAILFROM'] : '',
			'from_name' => isset( $parsed['EMAILFROMNAME'] ) ? $parsed['EMAILFROMNAME'] : '',
			'cc'        => isset( $parsed['EMAILCC'] ) ? $parsed['EMAILCC'] : '',
			'bcc'       => isset( $parsed['EMAILBCC'] ) ? $parsed['EMAILBCC'] : '',
			'subject'   => isset( $parsed['EMAILSUBJECT'] ) ? $parsed['EMAILSUBJECT'] : '',
			'body'      => $body,
			'content'   => $this->get_content_type(),
			'charset'   => $this->get_charset(),
		);

		$this->set_mail_values( $data );

		$mailed = $this->send_email();

		// Set $this->set_error_message(); and complete the action automatically. May be use return true / false.
		if ( false === $mailed && ! empty( $this->get_error_message() ) ) {

			$error_message = $this->get_error_message();

			$action_data['complete_with_errors'] = true;

			Automator()->complete->action( $user_id, $action_data, $recipe_id, $error_message );

			return;
		}

		Automator()->complete->action( $user_id, $action_data, $recipe_id );

	}

	/**
	 * @param $group_id
	 *
	 * @return string
	 */
	public function get_group_admin_emails( $group_id = 0 ) {

		if ( ! function_exists( 'learndash_get_groups_administrators' ) ) {

			return array();

		}

		$group_admins = array();

		$ld_group_admins = learndash_get_groups_administrators( $group_id, true );

		if ( ! empty( $ld_group_admins ) ) {

			foreach ( $ld_group_admins as $admin ) {

				$group_admins[] = $admin->user_email;

			}
		}

		return implode( ',', $group_admins );

	}

	/**
	 * @return array
	 */
	public function get_learndash_groups() {

		if ( ! function_exists( 'learndash_get_groups' ) ) {

			return array();

		}

		$groups = array();

		$learndash_groups = learndash_get_groups();

		foreach ( $learndash_groups as $group ) {

			$groups[ $group->ID ] = $group->post_title;

		}

		return $groups;

	}

}
