<?php

namespace Uncanny_Automator_Pro;

/**
 * Class WC_ORDER_PAYMENT_FAIL
 *
 * @package Uncanny_Automator_Pro
 */
class WC_ORDER_PAYMENT_FAIL {

	/**
	 * Integration code
	 *
	 * @var string
	 */
	public static $integration = 'WC';

	/**
	 * @var string
	 */
	private $trigger_code;

	/**
	 * @var string
	 */
	private $trigger_meta;

	/**
	 * Set up Automator trigger constructor.
	 */
	public function __construct() {
		$this->trigger_code = 'ORDERPAYMENTFAIL';
		$this->trigger_meta = 'ORDERFAIL';
		$this->define_trigger();
	}

	/**
	 * Define and register the trigger by pushing it into the Automator object
	 */
	public function define_trigger() {
		$trigger = array(
			'author'              => Automator()->get_author_name( $this->trigger_code ),
			'support_link'        => Automator()->get_author_support_link( $this->trigger_code, 'integration/woocommerce/' ),
			'is_pro'              => true,
			'type'                => 'anonymous',
			'integration'         => self::$integration,
			'code'                => $this->trigger_code,
			/* translators: Anonymous trigger - WooCommerce */
			'sentence'            => esc_attr__( 'A payment fails on an order', 'uncanny-automator-pro' ),
			/* translators: Anonymous trigger - WooCommerce */
			'select_option_name'  => esc_attr__( 'A payment fails on an order', 'uncanny-automator-pro' ),
			'action'              => 'woocommerce_order_status_changed',
			'priority'            => 10,
			'accepted_args'       => 4,
			'validation_function' => array( $this, 'order_payment_fails' ),
		);

		Automator()->register->trigger( $trigger );
	}

	/**
	 * Validation function when the trigger action is hit
	 *
	 * @param $order_id
	 */
	public function order_payment_fails( $order_id, $old_status, $new_status, $order ) {
		if ( ! $order_id ) {
			return;
		}

		if ( ! $order ) {
			return;
		}

		if ( 'failed' !== $new_status ) {
			return;
		}

		$user_id = $order->get_customer_id();
		$recipes = Automator()->get->recipes_from_trigger_code( $this->trigger_code );

		foreach ( $recipes as $recipe_id => $recipe ) {
			$pass_args = array(
				'code'           => $this->trigger_code,
				'meta'           => $this->trigger_meta,
				'user_id'        => $user_id,
				'ignore_post_id' => true,
			);

			$args = Automator()->maybe_add_trigger_entry( $pass_args, false );
			//Adding an action to save order id in trigger meta
			do_action( 'uap_wc_trigger_save_meta', $order_id, $recipe_id, $args, 'product' );

			if ( $args ) {
				foreach ( $args as $result ) {
					if ( true === $result['result'] ) {
						Automator()->maybe_trigger_complete( $result['args'] );
					}
				}
			}
		}
	}

}
